/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file VisualizationTool.C
 *    Implementation of VisualizationTool.
 *
 * @author Erik Einhorn
 * @date   2011/07/01
 */

#include <visualization/VisualizationTool.h>

#include <QMouseEvent>
#include <QToolTip>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

VisualizationTool::VisualizationTool() : 
	Authority(Authority::ANONYMOUS | Authority::INTERNAL)
{
	ok();
}

void VisualizationTool::init(IVisualizationSite* site)
{
	if (!isValid() ) {
		checkin("/","VisualizationTool");
		start();
	}
}

Authority& VisualizationTool::getAuthority()
{
	if (!isValid())
		MIRA_THROW(XRuntime, "Authority not initialized yet")

	return *this;
}


void VisualizationTool::updateToolTip(QMouseEvent* e, const QString& text)
{
	// Qt refuses to draw tooltip at new position if text stays the same.
	// So we apply a little magic here
	if (text == mLastToolTip)
		mLastToolTip = text + " ";
	else
		mLastToolTip = text;
	QToolTip::showText(e->globalPos(), mLastToolTip);
}

///////////////////////////////////////////////////////////////////////////////

}

MIRA_CLASS_SERIALIZATION(mira::VisualizationTool, mira::Object)
